@@ -273,3 +273,23 @@ def addModuleInfluencingCondition(
 
 def getModuleInfluences(module_name):
     return module_influencing_plugins.get(module_name, ())
+
+
+# Information about how long the optimization took.
+module_timing_infos = {}
+
+ModuleOptimizationTimingInfo = collections.namedtuple(
+    "ModuleOptimizationTimingInfo", ("pass_number", "time_used")
+)
+
+
+def addModuleOptimizationTimeInformation(module_name, pass_number, time_used):
+    module_timing_info = list(module_timing_infos.get(module_name, []))
+    module_timing_info.append(
+        ModuleOptimizationTimingInfo(pass_number=pass_number, time_used=time_used)
+    )
+    module_timing_infos[module_name] = tuple(module_timing_info)
+
+
+def getModuleOptimizationTimingInfos(module_name):
+    return module_timing_infos.get(module_name, ())
