@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,16 +24,20 @@ whose implementation lives here. The creation itself also lives here.
 
 from nuitka.PythonVersions import python_version
 
-from .ExpressionBases import ExpressionChildHavingBase
+from .ChildrenHavingMixins import ChildHavingGeneratorRefMixin
+from .ExpressionBases import ExpressionBase, ExpressionNoSideEffectsMixin
 from .FunctionNodes import ExpressionFunctionEntryPointBase
 from .IndicatorMixins import MarkUnoptimizedFunctionIndicatorMixin
 from .ReturnNodes import StatementReturn, StatementReturnNone
 
 
-class ExpressionMakeGeneratorObject(ExpressionChildHavingBase):
+class ExpressionMakeGeneratorObject(
+    ExpressionNoSideEffectsMixin, ChildHavingGeneratorRefMixin, ExpressionBase
+):
+
     kind = "EXPRESSION_MAKE_GENERATOR_OBJECT"
 
-    named_child = "generator_ref"
+    named_children = ("generator_ref",)
 
     __slots__ = ("variable_closure_traces",)
 
@@ -42,9 +46,9 @@ class ExpressionMakeGeneratorObject(ExpressionChildHavingBase):
             generator_ref.getFunctionBody().isExpressionGeneratorObjectBody()
         ), generator_ref
 
-        ExpressionChildHavingBase.__init__(
-            self, value=generator_ref, source_ref=source_ref
-        )
+        ChildHavingGeneratorRefMixin.__init__(self, generator_ref=generator_ref)
+
+        ExpressionBase.__init__(self, source_ref)
 
         self.variable_closure_traces = None
 
@@ -65,14 +69,6 @@ class ExpressionMakeGeneratorObject(ExpressionChildHavingBase):
         # TODO: Generator body may know something too.
         return self, None, None
 
-    @staticmethod
-    def mayRaiseException(exception_type):
-        return False
-
-    @staticmethod
-    def mayHaveSideEffects():
-        return False
-
     def getClosureVariableVersions(self):
         return self.variable_closure_traces
 
@@ -82,8 +78,15 @@ class ExpressionGeneratorObjectBody(
 ):
     kind = "EXPRESSION_GENERATOR_OBJECT_BODY"
 
+    __slots__ = (
+        "unoptimized_locals",
+        "unqualified_exec",
+        "needs_generator_return_exit",
+        "qualname_provider",
+    )
+
     if python_version >= 0x340:
-        qualname_setup = None
+        __slots__ += ("qualname_setup",)
 
     def __init__(self, provider, name, code_object, flags, auto_release, source_ref):
         ExpressionFunctionEntryPointBase.__init__(
@@ -103,6 +106,9 @@ class ExpressionGeneratorObjectBody(
 
         self.trace_collection = None
 
+        if python_version >= 0x340:
+            self.qualname_setup = None
+
     def getFunctionName(self):
         return self.name
 
@@ -145,7 +151,7 @@ class StatementGeneratorReturn(StatementReturn):
         if expression.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
 
-        if expression.willRaiseException(BaseException):
+        if expression.willRaiseAnyException():
             from .NodeMakingHelpers import (
                 makeStatementExpressionOnlyReplacementNode,
             )
