@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,7 +29,8 @@ from nuitka import PythonOperators
 from nuitka.Errors import NuitkaAssumptionError
 from nuitka.PythonVersions import python_version
 
-from .ExpressionBases import ExpressionChildrenHavingBase
+from .ChildrenHavingMixins import ChildrenHavingLeftRightMixin
+from .ExpressionBases import ExpressionBase
 from .NodeMakingHelpers import (
     makeRaiseExceptionReplacementExpressionFromInstance,
     wrapExpressionWithSideEffects,
@@ -54,7 +55,7 @@ class ExpressionPropertiesFromTypeShapeMixin(object):
 
 
 class ExpressionOperationBinaryBase(
-    ExpressionPropertiesFromTypeShapeMixin, ExpressionChildrenHavingBase
+    ExpressionPropertiesFromTypeShapeMixin, ChildrenHavingLeftRightMixin, ExpressionBase
 ):
     """Base class for all binary operation expression."""
 
@@ -64,9 +65,9 @@ class ExpressionOperationBinaryBase(
     nice_children = tuple(child_name + " operand" for child_name in named_children)
 
     def __init__(self, left, right, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self, values={"left": left, "right": right}, source_ref=source_ref
-        )
+        ChildrenHavingLeftRightMixin.__init__(self, left=left, right=right)
+
+        ExpressionBase.__init__(self, source_ref)
 
         # Question might be asked early on, later this is cached from last computation.
         self.type_shape = tshape_unknown
@@ -87,7 +88,7 @@ class ExpressionOperationBinaryBase(
     def markAsInplaceSuspect(self):
         self.inplace_suspect = True
 
-    def unmarkAsInplaceSuspect(self):
+    def removeMarkAsInplaceSuspect(self):
         self.inplace_suspect = False
 
     def isInplaceSuspect(self):
@@ -225,14 +226,14 @@ class ExpressionOperationBinaryBase(
 
         return self, None, None
 
-    def canPredictIterationValues(self):
+    @staticmethod
+    def canPredictIterationValues():
         # TODO: Actually we could very well, esp. for sequence repeats.
-        # pylint: disable=no-self-use
         return False
 
 
 class ExpressionOperationAddMixin(object):
-    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    # Mixins are not allowed to specify slots, pylint: disable=assigning-non-slot
     __slots__ = ()
 
     def getValueShape(self):
@@ -289,7 +290,7 @@ class ExpressionOperationBinarySub(ExpressionOperationBinaryBase):
 
 
 class ExpressionOperationMultMixin(object):
-    # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
+    # Mixins are not allowed to specify slots, pylint: disable=assigning-non-slot
     __slots__ = ()
 
     def getValueShape(self):
@@ -490,7 +491,38 @@ class ExpressionOperationBinaryPow(ExpressionOperationBinaryBase):
         return left_shape.getOperationBinaryPowShape(right_shape)
 
 
-class ExpressionOperationBinaryLshift(ExpressionOperationBinaryBase):
+class ExpressionOperationLshiftMixin(object):
+    # Mixins are not allowed to specify slots, pylint: disable=assigning-non-slot
+    __slots__ = ()
+
+    def getValueShape(self):
+        return self.shape
+
+    def _isTooLarge(self):
+        if self.subnode_right.isNumberConstant():
+            if self.subnode_left.isNumberConstant():
+                # Estimate with logarithm, if the result of number
+                # calculations is computable with acceptable effort,
+                # otherwise, we will have to do it at runtime.
+                left_value = self.subnode_left.getCompileTimeConstant()
+
+                if left_value != 0:
+                    right_value = self.subnode_right.getCompileTimeConstant()
+
+                    # More than a typical shift, most likely a stupid test.
+                    if right_value > 64:
+                        self.shape = ShapeLargeConstantValue(
+                            size=None, shape=tshape_int_or_long
+                        )
+
+                        return True
+
+        return False
+
+
+class ExpressionOperationBinaryLshift(
+    ExpressionOperationLshiftMixin, ExpressionOperationBinaryBase
+):
     kind = "EXPRESSION_OPERATION_BINARY_LSHIFT"
 
     operator = "LShift"
