@@ -44,6 +44,7 @@ from nuitka.nodes.shapes.StandardShapes import (
     tshape_uninitialized,
     tshape_unknown,
 )
+from nuitka.Tracing import my_print
 from nuitka.utils.InstanceCounters import (
     counted_del,
     counted_init,
@@ -88,6 +89,9 @@ class ValueTraceBase(object):
     def __repr__(self):
         return "<%s of %s>" % (self.__class__.__name__, self.owner.getCodeName())
 
+    def dump(self, indent):
+        my_print("%s%s %s:" % (indent, self.__class__.__name__, id(self)))
+
     def getOwner(self):
         return self.owner
 
@@ -183,6 +187,10 @@ class ValueTraceBase(object):
 
         return None
 
+    @staticmethod
+    def hasShapeListExact():
+        return False
+
     @staticmethod
     def hasShapeDictionaryExact():
         return False
@@ -276,9 +284,6 @@ class ValueTraceUninitialized(ValueTraceUnassignedBase):
     def isTraceThatNeedsEscape():
         return False
 
-    def dump(self, indent):
-        print("%s%s %s:" % (indent, self.__class__.__name__, id(self)))
-
 
 class ValueTraceDeleted(ValueTraceUnassignedBase):
     """Trace caused by a deletion."""
@@ -473,6 +478,16 @@ class ValueTraceEscaped(ValueTraceUnknown):
     def getAttributeNodeVeryTrusted(self):
         return self.previous.getAttributeNodeVeryTrusted()
 
+    # For escaped values, these do not have to forget their shape, the mutable ones
+    # and the only ones affected.
+    def hasShapeListExact(self):
+        trusted_node = self.previous.getAttributeNodeTrusted()
+        return trusted_node is not None and trusted_node.hasShapeListExact()
+
+    def hasShapeDictionaryExact(self):
+        trusted_node = self.previous.getAttributeNodeTrusted()
+        return trusted_node is not None and trusted_node.hasShapeDictionaryExact()
+
 
 class ValueTraceAssign(ValueTraceBase):
     __slots__ = ("assign_node",)
@@ -513,6 +528,9 @@ class ValueTraceAssign(ValueTraceBase):
     def getAssignNode(self):
         return self.assign_node
 
+    def hasShapeListExact(self):
+        return self.assign_node.subnode_source.hasShapeListExact()
+
     def hasShapeDictionaryExact(self):
         return self.assign_node.subnode_source.hasShapeDictionaryExact()
 
@@ -540,11 +558,9 @@ class ValueTraceAssign(ValueTraceBase):
             return None
 
     def getAttributeNodeVeryTrusted(self):
-        source_node = self.assign_node.subnode_source
-
         # Hard imports typically.
-        if source_node.hasVeryTrustedValue():
-            return source_node
+        if self.assign_node.hasVeryTrustedValue():
+            return self.assign_node.subnode_source
         else:
             return None
 
@@ -606,7 +622,7 @@ class ValueTraceMergeBase(ValueTraceBase):
         self.merge_usage_count += 1
 
     def dump(self, indent):
-        print("%s%s %s:" % (indent, self.__class__.__name__, id(self)))
+        ValueTraceBase.dump(self, indent)
 
         for trace in self.previous:
             trace.dump(indent + "  ")
@@ -710,6 +726,9 @@ class ValueTraceMerge(ValueTraceMergeBase):
 
         return True
 
+    def hasShapeListExact(self):
+        return all(previous.hasShapeListExact() for previous in self.previous)
+
     def hasShapeDictionaryExact(self):
         return all(previous.hasShapeDictionaryExact() for previous in self.previous)
 
