@@ -29,12 +29,14 @@ it being used.
 import ast
 import functools
 import inspect
+import os
 import sys
 from collections import namedtuple
 
 from nuitka.__past__ import getMetaClassBase
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.freezer.IncludedDataFiles import (
+    decodeDataFileTags,
     makeIncludedDataDirectory,
     makeIncludedDataFile,
     makeIncludedEmptyDirectory,
@@ -49,24 +51,102 @@ from nuitka.ModuleRegistry import (
     addModuleInfluencingCondition,
     getModuleInclusionInfoByName,
 )
-from nuitka.Options import isStandaloneMode, shallMakeModule
+from nuitka.Options import (
+    hasPythonFlagNoAnnotations,
+    hasPythonFlagNoAsserts,
+    hasPythonFlagNoDocStrings,
+    isStandaloneMode,
+    shallCreateAppBundle,
+    shallMakeModule,
+    shallShowExecutedCommands,
+)
 from nuitka.PythonFlavors import isAnacondaPython, isDebianPackagePython
 from nuitka.PythonVersions import getSupportedPythonVersions, python_version
 from nuitka.Tracing import plugins_logger
 from nuitka.utils.Execution import NuitkaCalledProcessError, check_output
-from nuitka.utils.ModuleNames import ModuleName
+from nuitka.utils.ModuleNames import (
+    ModuleName,
+    makeTriggerModuleName,
+    post_module_load_trigger_name,
+    pre_module_load_trigger_name,
+)
 from nuitka.utils.SharedLibraries import locateDLL, locateDLLsInDirectory
-from nuitka.utils.Utils import isLinux, isMacOS, isWin32Windows
+from nuitka.utils.Utils import (
+    getArchitecture,
+    isLinux,
+    isMacOS,
+    isWin32Windows,
+)
 
-warned_unused_plugins = set()
+_warned_unused_plugins = set()
 
-# Trigger names for shared use.
-post_module_load_trigger_name = "postLoad"
-pre_module_load_trigger_name = "preLoad"
+# TODO: Could share data cache with meta data nodes
+_package_versions = {}
 
 
-def makeTriggerModuleName(module_name, trigger_name):
-    return ModuleName(module_name + "-" + trigger_name)
+def _convertVersionToTuple(version_str):
+    def numberize(v):
+        # For now, we ignore rc/post stuff, hoping it doesn't matter for us.
+        return int("".join(d for d in v if d.isdigit()))
+
+    return tuple(numberize(d) for d in version_str.split("."))
+
+
+def _getPackageNameFromDistributionName(distribution_name):
+    if distribution_name in ("opencv-python", "opencv-python-headless"):
+        return "cv2"
+    elif distribution_name == "pyobjc":
+        return "objc"
+    else:
+        return distribution_name
+
+
+def _getPackageVersion(distribution_name):
+    if distribution_name not in _package_versions:
+        try:
+            if python_version >= 0x380:
+                from importlib.metadata import version
+            else:
+                from importlib_metadata import version
+
+            result = _convertVersionToTuple(version(distribution_name))
+        except ImportError:
+            try:
+                from pkg_resources import (
+                    DistributionNotFound,
+                    get_distribution,
+                )
+            except ImportError:
+                result = None
+            else:
+                try:
+                    result = _convertVersionToTuple(
+                        get_distribution(distribution_name).version
+                    )
+                except DistributionNotFound:
+                    result = None
+
+        if result is None:
+            # Fallback if nothing is available, which may happen if no package is installed,
+            # but only source code is found.
+            try:
+                result = _convertVersionToTuple(
+                    __import__(
+                        _getPackageNameFromDistributionName(distribution_name)
+                    ).__version__
+                )
+            except ImportError:
+                result = None
+
+        _package_versions[distribution_name] = result
+
+    return _package_versions[distribution_name]
+
+
+def _isPluginActive(plugin_name):
+    from .Plugins import getUserActivatedPluginNames
+
+    return plugin_name in getUserActivatedPluginNames()
 
 
 class NuitkaPluginBase(getMetaClassBase("Plugin")):
@@ -139,33 +219,19 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
 
     @classmethod
     def isDeprecated(cls):
+        """Is this a deprecated plugin, i.e. one that has no use anymore."""
         return False
 
+    @classmethod
+    def isDetector(cls):
+        """Is this a detection plugin, i.e. one which is only there to inform."""
+        return hasattr(cls, "detector_for")
+
     @classmethod
     def addPluginCommandLineOptions(cls, group):
         # Call group.add_option() here.
         pass
 
-    @classmethod
-    def getPluginDefaultOptionValues(cls):
-        """This method is used to get a values to use as defaults.
-
-        Since the defaults are in the command line options, we call
-        that and extract them.
-        """
-
-        from optparse import OptionGroup, OptionParser
-
-        parser = OptionParser()
-        group = OptionGroup(parser, "Pseudo Target")
-        cls.addPluginCommandLineOptions(group)
-
-        result = {}
-        for option in group.option_list:
-            result[option.dest] = option.default
-
-        return result
-
     def isRequiredImplicitImport(self, module, full_name):
         """Indicate whether an implicitly imported module should be accepted.
 
@@ -334,6 +400,20 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
 
+    def getPackageExtraScanPaths(self, package_name, package_dir):
+        """Provide other directories to consider submodules to live in.
+
+        Args:
+            module_name: full module name
+            package_dir: directory of the package
+
+        Returns:
+            Iterable list of directories, non-existent ones are ignored.
+        """
+
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return ()
+
     def onModuleEncounter(self, module_name, module_filename, module_kind):
         """Help decide whether to include a module.
 
@@ -347,13 +427,17 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
 
-    def onModuleRecursion(self, module_name, module_filename, module_kind):
+    def onModuleRecursion(
+        self, module_name, module_filename, module_kind, using_module, source_ref
+    ):
         """React to recursion to a module coming up.
 
         Args:
             module_name: full module name
             module_filename: filename
             module_kind: one of "py", "extension" (shared library)
+            using_module: module object that does the usage (None if it is a user choice)
+            source_ref: code making the import (None if it is a user choice)
         Returns:
             None
         """
@@ -382,12 +466,44 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             prepare.
         """
 
+    def onModuleCompleteSetGUI(self, module_set, plugin_binding_name):
+        from .Plugins import getOtherGUIBindingNames, getQtBindingNames
+
+        for module in module_set:
+            module_name = module.getFullName()
+
+            if module_name == plugin_binding_name:
+                continue
+
+            if module_name in getOtherGUIBindingNames():
+                if plugin_binding_name in getQtBindingNames():
+                    recommendation = "Use '--nofollow-import-to=%s'" % module_name
+
+                    if module_name in getQtBindingNames():
+                        problem = "conflicts with"
+                    else:
+                        problem = "is redundant with"
+                else:
+                    recommendation = "Use '--enable-plugin=no-qt'"
+                    problem = "is redundant with"
+
+                self.warning(
+                    """\
+Unwanted import of '%(unwanted)s' that %(problem)s '%(binding_name)s' encountered. \
+%(recommendation)s or uninstall it for best compatibility with pure Python execution."""
+                    % {
+                        "unwanted": module_name,
+                        "binding_name": plugin_binding_name,
+                        "recommendation": recommendation,
+                        "problem": problem,
+                    }
+                )
+
     @staticmethod
     def locateModule(module_name):
         """Provide a filename / -path for a to-be-imported module.
 
         Args:
-            importing: module object that asked for it (tracing only)
             module_name: (str or ModuleName) full name of module
         Returns:
             filename for module
@@ -395,7 +511,7 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
 
         from nuitka.importing.Importing import locateModule
 
-        _module_name, module_filename, _finding = locateModule(
+        _module_name, module_filename, _module_kind, _finding = locateModule(
             module_name=ModuleName(module_name), parent_package=None, level=0
         )
 
@@ -429,22 +545,24 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         """
         return locateDLLsInDirectory(directory)
 
-    def makeDllEntryPoint(self, source_path, dest_path, package_name):
+    def makeDllEntryPoint(self, source_path, dest_path, package_name, reason):
         """Create an entry point, as expected to be provided by getExtraDlls."""
         return makeDllEntryPoint(
             logger=self,
             source_path=source_path,
             dest_path=dest_path,
             package_name=package_name,
+            reason=reason,
         )
 
-    def makeExeEntryPoint(self, source_path, dest_path, package_name):
+    def makeExeEntryPoint(self, source_path, dest_path, package_name, reason):
         """Create an entry point, as expected to be provided by getExtraDlls."""
         return makeExeEntryPoint(
             logger=self,
             source_path=source_path,
             dest_path=dest_path,
             package_name=package_name,
+            reason=reason,
         )
 
     def reportFileCount(self, module_name, count, section=None):
@@ -452,7 +570,7 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             msg = "Found %d %s DLLs from %s%s installation." % (
                 count,
                 "file" if count < 2 else "files",
-                "" if not section else (" '%s' " % section),
+                "" if not section else ("'%s' " % section),
                 module_name.asString(),
             )
 
@@ -496,6 +614,17 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return ()
 
+    def getModuleSysPathAdditions(self, module_name):
+        """Provide a list of directories, that should be considered in 'PYTHONPATH' when this module is used.
+
+        Args:
+            module_name: name of a package or module
+        Returns:
+            iterable of paths
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return ()
+
     def removeDllDependencies(self, dll_filename, dll_filenames):
         """Yield any DLLs / shared libraries not to be included in distribution.
 
@@ -530,6 +659,24 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             tags=tags,
         )
 
+    def makeIncludedAppBundleResourceFile(
+        self, source_path, dest_path, reason, tags=""
+    ):
+        tags = decodeDataFileTags(tags)
+        tags.add("framework_resource")
+
+        assert isMacOS() and shallCreateAppBundle()
+
+        # The default dest path root is the "Contents" folder
+        dest_path = os.path.join("..", "Resources", dest_path)
+
+        return self.makeIncludedDataFile(
+            source_path=source_path,
+            dest_path=dest_path,
+            reason=reason,
+            tags=tags,
+        )
+
     def makeIncludedGeneratedDataFile(self, data, dest_path, reason, tags=""):
         return makeIncludedGeneratedDataFile(
             data=data, dest_path=dest_path, reason=reason, tracer=self, tags=tags
@@ -637,6 +784,18 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
 
+    def onStandaloneBinary(self, filename):
+        """Called after successfully creating a standalone binary.
+
+        Args:
+            filename: the created standalone binary
+
+        Returns:
+            None
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
     def onFinalResult(self, filename):
         """Called after successfully finishing a compilation.
 
@@ -695,7 +854,20 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             None for no defines, otherwise dictionary of key to be
             defined, and non-None values if any, i.e. no "-Dkey" only
         """
+        # Virtual method, pylint: disable=no-self-use
+        # spell-checker: ignore -Dkey
+        return None
 
+    def getBuildDefinitions(self):
+        """Decide C source defines to be used in compilation.
+
+        Notes:
+            Make sure to use a namespace for your defines, and prefer
+            `getPreprocessorSymbols` if you can.
+
+        Returns:
+            dict or None for no values
+        """
         # Virtual method, pylint: disable=no-self-use
         return None
 
@@ -760,8 +932,8 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         Returns:
             None
         """
-        if self.plugin_name not in warned_unused_plugins:
-            warned_unused_plugins.add(self.plugin_name)
+        if self.plugin_name not in _warned_unused_plugins:
+            _warned_unused_plugins.add(self.plugin_name)
 
             plugins_logger.warning(
                 "Use '--enable-plugin=%s' for: %s" % (self.plugin_name, message)
@@ -812,23 +984,29 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             query_codes.append('print("-" * 27)')
 
         if type(setup_codes) is str:
-            setup_codes = setup_codes.split("\n")
+            setup_codes = setup_codes.splitlines()
+
+        if not setup_codes:
+            setup_codes = ["pass"]
 
         cmd = r"""\
 from __future__ import print_function
 from __future__ import absolute_import
 
 try:
-    %(setup_codes)s
+%(setup_codes)s
 except ImportError:
     import sys
     sys.exit(38)
 %(query_codes)s
 """ % {
-            "setup_codes": "\n   ".join(setup_codes),
+            "setup_codes": "\n".join("   %s" % line for line in setup_codes),
             "query_codes": "\n".join(query_codes),
         }
 
+        if shallShowExecutedCommands():
+            self.info("Executing query command:\n%s" % cmd)
+
         try:
             feedback = check_output([sys.executable, "-c", cmd])
         except NuitkaCalledProcessError as e:
@@ -839,6 +1017,9 @@ except ImportError:
         if str is not bytes:  # We want to work with strings, that's hopefully OK.
             feedback = feedback.decode("utf8")
 
+        if shallShowExecutedCommands():
+            self.info("Result of query command:\n%s" % feedback)
+
         # Ignore Windows newlines difference.
         feedback = [line.strip() for line in feedback.splitlines()]
 
@@ -874,15 +1055,26 @@ except ImportError:
 
         This must be used to invalidate cache results, e.g. when using the
         onFunctionBodyParsing function, and other things, that do not directly
-        affect the source code.
+        affect the source code. By default a plugin being enabled changes the
+        result unless it makes it clear that is not the case.
         """
-        # Virtual method, pylint: disable=no-self-use,unused-argument
-        return ()
+        # Virtual method, pylint: disable=unused-argument
+        return self.plugin_name
+
+    @staticmethod
+    def getPackageVersion(distribution_name):
+        """Provide package version of a distribution."""
+        return _getPackageVersion(distribution_name)
 
     def evaluateCondition(self, full_name, condition, control_tags=()):
         # Note: Caching makes no sense yet, this should all be very fast and
         # cache themselves. TODO: Allow plugins to contribute their own control
         # tag values during creation and during certain actions.
+        if condition == "True":
+            return True
+        if condition == "False":
+            return False
+
         context = TagContext(logger=self, full_name=full_name)
         context.update(control_tags)
 
@@ -897,11 +1089,24 @@ except ImportError:
                 "module_mode": shallMakeModule(),
                 # TODO: Allow to provide this.
                 "deployment": False,
-                # Module dependent
-                "main_module": full_name.getBasename() == "__main__",
+                # Querying package versions.
+                "version": _getPackageVersion,
+                "plugin": _isPluginActive,
+                "no_asserts": hasPythonFlagNoAsserts(),
+                "no_docstrings": hasPythonFlagNoDocStrings(),
+                "no_annotations": hasPythonFlagNoAnnotations(),
             }
         )
 
+        if isWin32Windows():
+            context.update(
+                {
+                    "arch_x86": getArchitecture() == "x86",
+                    "arch_amd64": getArchitecture() == "x86_64",
+                    "arch_arm64": getArchitecture() == "arm64",
+                }
+            )
+
         versions = getSupportedPythonVersions()
 
         for version in versions:
@@ -916,7 +1121,13 @@ except ImportError:
         context["python3_or_higher"] = python_version >= 0x300
 
         # We trust the yaml files, pylint: disable=eval-used
-        result = eval(condition, context)
+        try:
+            result = eval(condition, context)
+        except Exception as e:  # Catch all the things, pylint: disable=broad-except
+            self.sysexit(
+                "Error, failed to evaluate condition '%s' in this context, exception was '%s'."
+                % (condition, e)
+            )
 
         if type(result) is not bool:
             self.sysexit(
@@ -935,8 +1146,14 @@ except ImportError:
         return result
 
     @classmethod
-    def warning(cls, message):
-        plugins_logger.warning(cls.plugin_name + ": " + message)
+    def warning(cls, message, **kwargs):
+        # Doing keyword only arguments manually, to keep older Python compatibility, and avoid
+        # user errors still.
+        mnemonic = kwargs.pop("mnemonic", None)
+        if kwargs:
+            plugins_logger.sysexit("Illegal arguments for self.warning")
+
+        plugins_logger.warning(cls.plugin_name + ": " + message, mnemonic=mnemonic)
 
     @classmethod
     def info(cls, message):
@@ -982,6 +1199,6 @@ class TagContext(dict):
                 return False
 
             self.logger.sysexit(
-                "Control tag '%s' in module configuration of '%s' is unknown."
+                "Identifier '%s' in 'when' configuration of module '%s' is unknown."
                 % (key, self.full_name)
             )
