@@ -37,9 +37,10 @@ from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
 from nuitka.containers.OrderedDicts import OrderedDict
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.freezer.IncludedDataFiles import IncludedDataFile
+from nuitka.freezer.IncludedEntryPoints import IncludedEntryPoint
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.Tracing import plugins_logger, printLine
-from nuitka.utils.FileOperations import putTextFileContents
+from nuitka.utils.FileOperations import makePath, putTextFileContents
 from nuitka.utils.Importing import importFileAsModule
 from nuitka.utils.ModuleNames import (
     ModuleName,
@@ -60,8 +61,11 @@ user_plugins = OrderedSet()
 
 # Trigger modules
 pre_modules = {}
+pre_modules_reasons = {}
 post_modules = {}
+post_modules_reasons = {}
 fake_modules = {}
+has_active_gui_toolkit_plugin = False
 
 
 def _addActivePlugin(plugin_class, args, force=False):
@@ -88,6 +92,12 @@ def _addActivePlugin(plugin_class, args, force=False):
 
     active_plugins[plugin_name] = plugin_instance
 
+    # Singleton, pylint: disable=global-statement
+    global has_active_gui_toolkit_plugin
+    has_active_gui_toolkit_plugin = has_active_gui_toolkit_plugin or getattr(
+        plugin_class, "plugin_gui_toolkit", False
+    )
+
 
 def getActivePlugins():
     """Return list of active plugins.
@@ -101,8 +111,6 @@ def getActivePlugins():
 
 
 def getActiveQtPlugin():
-    from .standard.PySidePyQtPlugin import getQtPluginNames
-
     for plugin_name in getQtPluginNames():
         if hasActivePlugin(plugin_name):
             if hasActivePlugin(plugin_name):
@@ -111,6 +119,26 @@ def getActiveQtPlugin():
     return None
 
 
+def getQtBindingNames():
+    return ("PySide", "PySide2", "PySide6", "PyQt4", "PyQt5", "PyQt6")
+
+
+def getOtherGUIBindingNames():
+    return ("wx", "tkinter", "Tkinter")
+
+
+def getQtPluginNames():
+    return tuple(qt_binding_name.lower() for qt_binding_name in getQtBindingNames())
+
+
+def getOtherGuiPluginNames():
+    return ("tk-inter",)
+
+
+def getGuiPluginNames():
+    return getQtPluginNames() + getOtherGuiPluginNames()
+
+
 def hasActivePlugin(plugin_name):
     """Decide if a plugin is active.
 
@@ -132,6 +160,19 @@ def hasActivePlugin(plugin_name):
     return not hasattr(plugin_instance, "detector_for")
 
 
+def getUserActivatedPluginNames():
+    for plugin_name, plugin_instance in active_plugins.items():
+        # Detectors do not count.
+        if hasattr(plugin_instance, "detector_for"):
+            continue
+
+        # Always activated does not count either
+        if plugin_instance.isAlwaysEnabled():
+            continue
+
+        yield plugin_name
+
+
 def getPluginClass(plugin_name):
     # First, load plugin classes, to know what we are talking about.
     loadPlugins()
@@ -183,7 +224,7 @@ def _loadPluginClassesFromPackage(scan_package):
         except Exception:
             if Options.is_non_debug:
                 plugins_logger.warning(
-                    "Problem loading plugin %r (%s), ignored. Use '--debug' to make it visible."
+                    "Problem loading plugin %r ('%s'), ignored. Use '--debug' to make it visible."
                     % (item.name, module_loader.get_filename())
                 )
                 continue
@@ -266,8 +307,6 @@ class Plugins(object):
 
     @staticmethod
     def _considerImplicitImports(plugin, module):
-        from nuitka.importing import Importing
-
         result = []
 
         def iterateModuleNames(value):
@@ -305,11 +344,7 @@ class Plugins(object):
                 continue
 
             try:
-                _module_name, module_filename, _finding = Importing.locateModule(
-                    module_name=full_name,
-                    parent_package=None,
-                    level=0,
-                )
+                module_filename = plugin.locateModule(full_name)
             except Exception:
                 plugin.warning(
                     "Problem locating '%s' for implicit imports of '%s'."
@@ -342,6 +377,8 @@ class Plugins(object):
         from nuitka.importing.Importing import getModuleNameAndKindFromFilename
 
         for full_name, module_filename in implicit_imports:
+
+            # TODO: The module_kind should be forwarded from previous in the class using locateModule code.
             _module_name2, module_kind = getModuleNameAndKindFromFilename(
                 module_filename
             )
@@ -359,6 +396,8 @@ class Plugins(object):
                     module_name=full_name,
                     module_filename=module_filename,
                     module_kind=module_kind,
+                    using_module=module,
+                    source_ref=module.getSourceReference(),
                     reason=reason,
                 )
 
@@ -402,7 +441,7 @@ class Plugins(object):
                 pre_modules[full_name],
                 using_module=module,
                 usage_tag="plugins",
-                reason="Not yet propagated by plugins.",
+                reason=" ".join(pre_modules_reasons[full_name]),
                 source_ref=module.source_ref,
             )
 
@@ -411,7 +450,7 @@ class Plugins(object):
                 module=post_modules[full_name],
                 using_module=module,
                 usage_tag="plugins",
-                reason="Not yet propagated by plugins.",
+                reason=" ".join(post_modules_reasons[full_name]),
                 source_ref=module.source_ref,
             )
 
@@ -475,18 +514,34 @@ class Plugins(object):
         """Ask plugins to provide extra DLLs.
 
         Notes:
-            These will be of type nuitka.freezer.IncludedEntryPoints.IncludedEntryPoint
-            and currently there is a backward compatibility for old style plugins that do
-            provide tuples of 3 elements. But plugins are really supposed to provide the
-            stuff created from factory functions for that type.
+            These will be of type IncludedEntryPoint or generators providing them, so
+            we get "yield from" effective with simple yield.
 
         """
 
         result = []
 
+        # TODO: This is probably something generic that only different
+        def _iterateExtraBinaries(plugin, value):
+            if value is None:
+                pass
+            elif isinstance(value, IncludedEntryPoint):
+                yield value
+            elif isinstance(value, (tuple, list)) or inspect.isgenerator(value):
+                for val in value:
+                    for v in _iterateExtraBinaries(plugin, val):
+                        yield v
+            else:
+                plugin.sysexit(
+                    "Returned object '%s' for module '%s' but should do 'IncludedEntryPoint' or generator."
+                    % (repr(value), module.asString())
+                )
+
         for plugin in getActivePlugins():
-            for extra_dll in plugin.getExtraDlls(module):
-                result.append(extra_dll)
+            for entry_point in _iterateExtraBinaries(
+                plugin, plugin.getExtraDlls(module)
+            ):
+                result.append(entry_point)
 
         return result
 
@@ -505,6 +560,27 @@ class Plugins(object):
 
         return result
 
+    sys_path_additions_cache = {}
+
+    @classmethod
+    def getModuleSysPathAdditions(cls, module_name):
+        """Provide a list of directories, that should be considered in 'PYTHONPATH' when this module is used.
+
+        Args:
+            module_name: name of a package or module
+        Returns:
+            iterable of paths
+        """
+
+        if module_name not in cls.sys_path_additions_cache:
+            cls.sys_path_additions_cache[module_name] = OrderedSet()
+
+            for plugin in getActivePlugins():
+                for dll_path in plugin.getModuleSysPathAdditions(module_name):
+                    cls.sys_path_additions_cache[module_name].add(dll_path)
+
+        return cls.sys_path_additions_cache[module_name]
+
     @staticmethod
     def removeDllDependencies(dll_filename, dll_filenames):
         """Create list of removable shared libraries by scanning through the plugins.
@@ -697,6 +773,7 @@ class Plugins(object):
         if pre_module_load_descriptions:
             total_code = []
             total_flags = OrderedSet()
+            reasons = []
 
             for plugin, pre_code, reason, flags in pre_module_load_descriptions:
                 if pre_code:
@@ -708,6 +785,7 @@ class Plugins(object):
 
                     total_code.append(pre_code)
                     total_flags.update(flags)
+                    reasons.append(reason)
 
             if total_code:
                 assert full_name not in pre_modules
@@ -718,10 +796,12 @@ class Plugins(object):
                     code="\n\n".join(total_code),
                     flags=total_flags,
                 )
+                pre_modules_reasons[full_name] = tuple(reasons)
 
         if post_module_load_descriptions:
             total_code = []
             total_flags = OrderedSet()
+            reasons = []
 
             for plugin, post_code, reason, flags in post_module_load_descriptions:
                 if post_code:
@@ -733,6 +813,7 @@ class Plugins(object):
 
                     total_code.append(post_code)
                     total_flags.update(flags)
+                    reasons.append(reason)
 
             if total_code:
                 assert full_name not in post_modules
@@ -743,6 +824,7 @@ class Plugins(object):
                     code="\n\n".join(total_code),
                     flags=total_flags,
                 )
+                post_modules_reasons[full_name] = reasons
 
         if fake_module_descriptions:
             fake_modules[full_name] = []
@@ -777,14 +859,17 @@ class Plugins(object):
         assert type(module_name) is ModuleName
         assert type(source_code) is str
 
+        contributing_plugins = OrderedSet()
+
         for plugin in getActivePlugins():
             new_source_code = plugin.onModuleSourceCode(module_name, source_code)
-            if new_source_code is not None:
+            if new_source_code is not None and new_source_code != source_code:
                 source_code = new_source_code
+                contributing_plugins.add(plugin)
 
             assert type(source_code) is str
 
-        return source_code
+        return source_code, contributing_plugins
 
     @staticmethod
     def onFrozenModuleSourceCode(module_name, is_package, source_code):
@@ -844,12 +929,16 @@ class Plugins(object):
         return result
 
     @staticmethod
-    def onModuleRecursion(module_name, module_filename, module_kind):
+    def onModuleRecursion(
+        module_name, module_filename, module_kind, using_module, source_ref
+    ):
         for plugin in getActivePlugins():
             plugin.onModuleRecursion(
                 module_name=module_name,
                 module_filename=module_filename,
                 module_kind=module_kind,
+                using_module=using_module,
+                source_ref=source_ref,
             )
 
     @staticmethod
@@ -947,6 +1036,38 @@ class Plugins(object):
 
         return cls.preprocessor_symbols
 
+    build_definitions = None
+
+    @classmethod
+    def getBuildDefinitions(cls):
+        """Let plugins provide C source defines to be used in compilation.
+
+        Notes:
+            The plugins can each contribute, but are hopefully using
+            a namespace for their defines. Only specific code sees these
+            if it chooses to include "build_definitions.h" file.
+
+        Returns:
+            OrderedDict() with keys and values."
+        """
+
+        if cls.build_definitions is None:
+            cls.build_definitions = OrderedDict()
+
+            for plugin in getActivePlugins():
+                value = plugin.getBuildDefinitions()
+
+                if value is not None:
+                    assert type(value) is dict, value
+
+                    # We order per plugin, but from the plugins themselves, lets just assume
+                    # unordered dict and achieve determinism by ordering the defines by name.
+                    for key, value in sorted(value.items()):
+                        # False alarm, pylint: disable=I0021,unsupported-assignment-operation
+                        cls.build_definitions[key] = value
+
+        return cls.build_definitions
+
     extra_include_directories = None
 
     @classmethod
@@ -972,7 +1093,7 @@ class Plugins(object):
         return cls.extra_include_directories
 
     @staticmethod
-    def getExtraCodeFiles():
+    def _getExtraCodeFiles(for_onefile):
         result = OrderedDict()
 
         for plugin in getActivePlugins():
@@ -984,6 +1105,12 @@ class Plugins(object):
                 # We order per plugin, but from the plugins, lets just take a dict
                 # and achieve determinism by ordering the files by name.
                 for key, value in sorted(value.items()):
+
+                    if (for_onefile and not "onefile_" in key) or (
+                        not for_onefile and "onefile_" in key
+                    ):
+                        continue
+
                     if not key.startswith("nuitka_"):
                         key = "plugin." + plugin.plugin_name + "." + key
 
@@ -992,6 +1119,23 @@ class Plugins(object):
 
         return result
 
+    @staticmethod
+    def writeExtraCodeFiles(onefile):
+        # Circular dependency.
+        from nuitka.tree.SourceHandling import writeSourceCode
+
+        source_dir = OutputDirectories.getSourceDirectoryPath(onefile=onefile)
+
+        for filename, source_code in Plugins._getExtraCodeFiles(onefile).items():
+            target_dir = os.path.join(source_dir, "plugins")
+
+            if not os.path.isdir(target_dir):
+                makePath(target_dir)
+
+            writeSourceCode(
+                filename=os.path.join(target_dir, filename), source_code=source_code
+            )
+
     extra_link_libraries = None
 
     @classmethod
@@ -1004,10 +1148,10 @@ class Plugins(object):
 
                 if value is not None:
                     if isinstance(value, basestring):
-                        cls.extra_link_libraries.add(value)
+                        cls.extra_link_libraries.add(os.path.normcase(value))
                     else:
                         for library_name in value:
-                            cls.extra_link_libraries.add(library_name)
+                            cls.extra_link_libraries.add(os.path.normcase(library_name))
 
         return cls.extra_link_libraries
 
@@ -1224,12 +1368,16 @@ def activatePlugins():
                 "Error, conflicting enable/disable of plug-in '%s'." % plugin_name
             )
 
+    plugin_detectors = OrderedSet()
+
     for (plugin_name, (plugin_class, plugin_detector)) in sorted(
         plugin_name2plugin_classes.items()
     ):
         if plugin_name in Options.getPluginsEnabled():
             if plugin_class.isAlwaysEnabled():
-                plugin_class.warning("Plugin is defined as always enabled.")
+                plugin_class.warning(
+                    "Plugin is defined as always enabled, no need to enable it."
+                )
 
             if plugin_class.isRelevant():
                 _addActivePlugin(plugin_class, args=True)
@@ -1250,27 +1398,25 @@ def activatePlugins():
             and Options.shallDetectMissingPlugins()
             and plugin_detector.isRelevant()
         ):
-            _addActivePlugin(plugin_detector, args=False)
+            plugin_detectors.add(plugin_detector)
 
     for plugin_class in user_plugins:
         _addActivePlugin(plugin_class, args=True)
 
-
-def lateActivatePlugin(plugin_name, option_values):
-    """Activate plugin after the command line parsing, expects options to be set."""
-
-    values = getPluginClass(plugin_name).getPluginDefaultOptionValues()
-    values.update(option_values)
-    setPluginOptions(plugin_name, values)
-
-    _addActivePlugin(getPluginClass(plugin_name), args=True, force=True)
+    # Suppress GUI toolkit detectors automatically.
+    for plugin_detector in plugin_detectors:
+        if (
+            not has_active_gui_toolkit_plugin
+            or plugin_detector.plugin_name not in getGuiPluginNames()
+        ):
+            _addActivePlugin(plugin_detector, args=False)
 
 
 def _addPluginCommandLineOptions(parser, plugin_class):
     plugin_name = plugin_class.plugin_name
 
     if plugin_name not in plugin_options:
-        option_group = OptionGroup(parser, "Plugin %s" % plugin_name)
+        option_group = OptionGroup(parser, "Plugin options of '%s'" % plugin_name)
         try:
             plugin_class.addPluginCommandLineOptions(option_group)
         except OptionConflictError as e:
