@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,7 +29,7 @@ import tempfile
 import threading
 import time
 from contextlib import contextmanager
-from optparse import OptionGroup, OptionParser
+from optparse import OptionParser
 
 from nuitka.__past__ import subprocess
 from nuitka.PythonVersions import (
@@ -38,23 +38,28 @@ from nuitka.PythonVersions import (
     isDebugPython,
 )
 from nuitka.Tracing import OurLogger, my_print
-from nuitka.tree.SourceReading import readSourceCodeFromFilename
+from nuitka.tree.SourceHandling import readSourceCodeFromFilename
 from nuitka.utils.AppDirs import getCacheDir
-from nuitka.utils.Execution import check_output, getNullInput, getNullOutput
+from nuitka.utils.Execution import (
+    check_output,
+    createProcess,
+    getNullInput,
+    getNullOutput,
+)
 from nuitka.utils.FileOperations import (
     areSamePaths,
     getExternalUsePath,
     getFileContentByLine,
     getFileContents,
     getFileList,
-    isPathBelowOrSameAs,
+    isFilenameSameAsOrBelowPath,
     makePath,
     openTextFile,
     removeDirectory,
 )
 from nuitka.utils.InstalledPythons import findInstalledPython
 from nuitka.utils.Jinja2 import getTemplate
-from nuitka.utils.Utils import getOS
+from nuitka.utils.Utils import getOS, isMacOS, isWin32Windows
 
 from .SearchModes import (
     SearchModeByPattern,
@@ -64,6 +69,8 @@ from .SearchModes import (
     SearchModeResume,
 )
 
+# spellchecker: ignore popenargs,pathsep,killpg
+
 test_logger = OurLogger("", base_style="blue")
 
 
@@ -73,9 +80,9 @@ def check_result(*popenargs, **kwargs):
 
     process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
     _unused_output, _unused_err = process.communicate()
-    retcode = process.poll()
+    ret_value = process.poll()
 
-    if retcode:
+    if ret_value:
         return False
     else:
         return True
@@ -159,7 +166,7 @@ def setup(suite="", needs_io_encoding=False, silent=False, go_main=True):
     if (
         len(os.environ["PYTHON"]) == 2
         and os.environ["PYTHON"].isdigit()
-        and os.name != "nt"
+        and not isWin32Windows()
     ):
 
         os.environ["PYTHON"] = "python%s.%s" % (
@@ -251,7 +258,7 @@ def convertUsing2to3(path, force=False):
 
     if use_binary:
         # On Windows, we cannot rely on 2to3 to be in the path.
-        if os.name == "nt":
+        if isWin32Windows():
             command = [
                 sys.executable,
                 os.path.join(os.path.dirname(sys.executable), "Tools/Scripts/2to3.py"),
@@ -267,7 +274,7 @@ def convertUsing2to3(path, force=False):
         check_output(command, stderr=getNullOutput())
 
     except subprocess.CalledProcessError:
-        if os.name == "nt":
+        if isWin32Windows():
             raise
 
         command[0:3] = ["2to3"]
@@ -388,7 +395,7 @@ def _removeCPythonTestSuiteDir():
         # call as last resort could be a good idea.
 
         # This seems to work for broken "lnk" files.
-        if os.name == "nt":
+        if isWin32Windows():
             os.system("rmdir /S /Q @test")
 
         if os.path.exists("@test"):
@@ -746,7 +753,7 @@ def createSearchMode():
 
     parser = OptionParser()
 
-    select_group = OptionGroup(parser, "Select Tests")
+    select_group = parser.add_option_group("Select Tests")
 
     select_group.add_option(
         "--pattern",
@@ -765,9 +772,9 @@ Execute only tests matching the pattern. Defaults to all tests.""",
 Execute all tests, continue execution even after failure of one.""",
     )
 
-    parser.add_option_group(select_group)
+    del select_group
 
-    debug_group = OptionGroup(parser, "Test features")
+    debug_group = parser.add_option_group("Test features")
 
     debug_group.add_option(
         "--debug",
@@ -788,7 +795,7 @@ Defaults to off.""",
 Defaults to off.""",
     )
 
-    parser.add_option_group(debug_group)
+    del debug_group
 
     options, positional_args = parser.parse_args()
 
@@ -840,7 +847,9 @@ def reportSkip(reason, dirname, filename):
     my_print("Skipped, %s (%s)." % (case, reason))
 
 
-def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr, explain=False):
+def executeReferenceChecked(
+    prefix, names, tests_skipped=(), tests_stderr=(), explain=False
+):
     gc.disable()
 
     extract_number = lambda name: int(name.replace(prefix, ""))
@@ -863,7 +872,7 @@ def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr, explain=
             my_print(name + ": SKIPPED (%s)" % tests_skipped[number])
             continue
 
-        # Avoid unraisable output.
+        # Avoid non-raisable output.
         try:
             if number in tests_stderr:
                 sys.stderr = getNullOutput()
@@ -1288,10 +1297,10 @@ def setupCacheHashSalt(test_code_path):
 
 
 def displayFolderContents(name, path):
-    test_logger.info("Listing of %s %r:" % (name, path))
+    test_logger.info("Listing of %s '%s':" % (name, path))
 
     if os.path.exists(path):
-        if os.name == "nt":
+        if isWin32Windows():
             command = "dir /b /s /a:-D %s" % path
         else:
             command = "ls -Rla %s" % path
@@ -1369,24 +1378,25 @@ class DelayedExecutionThread(threading.Thread):
         self.func()
 
 
-def executeAfterTimePassed(timeout, func):
+def executeAfterTimePassed(message, timeout, func):
+    test_logger.info(message % timeout)
+
     alarm = DelayedExecutionThread(timeout=timeout, func=func)
     alarm.start()
 
 
-def killProcess(name, pid):
+def killProcessGroup(process_name, pid):
     """Kill a process in a portable way.
 
     Right now SIGINT is used, unclear what to do on Windows
     with Python2 or non-related processes.
     """
 
-    if str is bytes and os.name == "nt":
-        test_logger.info("Using taskkill on test process %r." % name)
-        os.system("taskkill.exe /PID %d" % pid)
+    if isWin32Windows():
+        test_logger.sysexit("Error, cannot send kill signal on Windows")
     else:
-        test_logger.info("Killing test process %r." % name)
-        os.kill(pid, signal.SIGINT)
+        test_logger.info("Killing test process group '%s'." % process_name)
+        os.killpg(pid, signal.SIGINT)
 
 
 def checkLoadedFileAccesses(loaded_filenames, current_dir):
@@ -1405,7 +1415,7 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
         loaded_filename = os.path.normcase(loaded_filename)
         loaded_basename = os.path.basename(loaded_filename)
 
-        if os.name == "nt":
+        if isWin32Windows():
             if areSamePaths(
                 os.path.dirname(loaded_filename),
                 os.path.normpath(os.path.join(os.environ["SYSTEMROOT"], "System32")),
@@ -1461,7 +1471,7 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
             # Terminal info files are OK too.
             "/lib/terminfo",
         ):
-            if isPathBelowOrSameAs(ignored_dir, loaded_filename):
+            if isFilenameSameAsOrBelowPath(ignored_dir, loaded_filename):
                 ignore = False
                 break
         if not ignore:
@@ -1770,6 +1780,34 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
         if loaded_basename.upper() in ("MSVCRT.DLL", "MSVCR90.DLL"):
             continue
 
+        if isMacOS():
+            ignore = True
+            for ignored_dir in (
+                "/System/Library/PrivateFrameworks",
+                "/System/Library/CoreService",
+                "/System/Library/Frameworks/CoreFoundation.framework",
+                "/System/Library/dyld",
+                "/usr/lib/system/",
+            ):
+                if isFilenameSameAsOrBelowPath(ignored_dir, loaded_filename):
+                    ignore = False
+                    break
+            if not ignore:
+                continue
+
+            if loaded_filename == "/usr/libexec/rosetta/runtime":
+                continue
+
+            if loaded_filename in (
+                "/usr/lib/libSystem.B.dylib",
+                "/usr/lib/libc++.1.dylib",
+                "/usr/lib/libc++abi.dylib",
+                "/usr/lib/libfakelink.dylib",
+                "/usr/lib/liboah.dylib",
+                "/usr/lib/libobjc.A.dylib",
+            ):
+                continue
+
         illegal_accesses.append(orig_loaded_filename)
 
     return illegal_accesses
@@ -1799,3 +1837,81 @@ def getInstalledPythonVersion(python_version, must_exist):
         )
 
     return result
+
+
+_sys_path_path = None
+
+
+def getPythonSysPath():
+    global _sys_path_path  # singleton, pylint: disable=global-statement
+
+    if _sys_path_path is None:
+        _sys_path_path = check_output(
+            [
+                os.environ["PYTHON"],
+                "-c",
+                "import sys, os; print(os.pathsep.join(sys.path))",
+            ]
+        )
+
+        if str is not bytes:
+            _sys_path_path = _sys_path_path.decode("utf8")
+
+        _sys_path_path = _sys_path_path.strip()
+
+    return _sys_path_path
+
+
+_web_server_process = None
+_web_server_port = 27272
+_web_server_hostname = "localhost"
+
+
+def getLocalWebServerUrl():
+    return "http://%s:%d" % (_web_server_hostname, _web_server_port)
+
+
+def getLocalWebServerDir(base_dir):
+    global _web_server_process  # singleton, pylint: disable=global-statement
+
+    web_dir = os.path.join(getTempDir(), "local-web-server", base_dir)
+
+    if _web_server_process is None:
+        web_server_directory_supporting_pythons = ("3.10", "3.9", "3.8", "3.7")
+
+        web_server_python = findInstalledPython(
+            python_versions=web_server_directory_supporting_pythons,
+            module_name=None,
+            module_version=None,
+        )
+
+        if web_server_python is None:
+            return None
+
+        os.makedirs(web_dir)
+
+        command = [
+            web_server_python.getPythonExe(),
+            "-m",
+            "http.server",
+            "--bind",
+            _web_server_hostname,
+            "--dir",
+            web_dir,
+            str(_web_server_port),
+        ]
+
+        test_logger.my_print(" ".join(command))
+
+        _web_server_process = createProcess(
+            command,
+            stdout=sys.stderr,
+            stderr=sys.stderr,
+        )
+
+        def killWebServerProcess():
+            _web_server_process.kill()
+
+        atexit.register(killWebServerProcess)
+
+    return web_dir
