@@ -73,33 +73,16 @@ tempfile_re = re.compile(r"/tmp/tmp[a-z0-9_]*")
 logging_info_re = re.compile(r"^Nuitka.*?:INFO")
 logging_warning_re = re.compile(r"^Nuitka.*?:WARNING")
 
+# Python3.11 style traceback carets are not done by Nuitka (yet?)
+syntax_error_caret_re = re.compile(r"^\s*~*\^*~*$")
 
-def normalizeTimeDiff(outputStr):
-    """
-    use regular expression to normalize the time output
-
-    e.g.
-    =================== 1059 passed, 8 warnings in 7.99 seconds ===================
-
-    becomes
-    =================== 1059 passed, 8 warnings in x.xx seconds ===================
-    """
-
-    match = re.search(b"in [0-9]+.[0-9][0-9](s| seconds)", outputStr)
-    if match:
-        return (
-            outputStr[: match.start()] + b"in x.xx seconds" + outputStr[match.end() :]
-        )
-    return outputStr
+timing_re = re.compile(r"in [0-9]+.[0-9][0-9](s| seconds)")
 
 
 def makeDiffable(output, ignore_warnings, syntax_errors):
     # Of course many cases to deal with,
     # pylint: disable=too-many-branches,too-many-statements
 
-    # do not compare time differences
-    output = normalizeTimeDiff(output)
-
     result = []
 
     # Fix import "readline" because output sometimes starts with "\x1b[?1034h"
@@ -151,12 +134,16 @@ def makeDiffable(output, ignore_warnings, syntax_errors):
         if line.startswith("Nuitka:WARNING: Using very slow fallback for ordered sets"):
             continue
 
+        if syntax_error_caret_re.match(line):
+            continue
+
         line = instance_re.sub(r"at 0xxxxxxxxx\1", line)
         line = thread_re.sub(r"Thread 0xXXXXXXXX", line)
         line = compiled_types_re.sub(r"\1", line)
         line = global_name_error_re.sub(r"\1\2\3", line)
         line = module_repr_re.sub(r"\1xxxxx\2", line)
         line = non_ascii_error_rt.sub(r"\1 xxxx", line)
+        line = timing_re.sub(r"in x.xx seconds", line)
 
         # Windows has a different "os.path", update according to it.
         line = line.replace("ntpath", "posixpath")
@@ -225,6 +212,15 @@ exceeded while calling a Python object' in \
         if "clcache: persistent json file" in line or "clcache: manifest file" in line:
             continue
 
+        # Ignore manual error message of CPython 3.11 that is different from the generic one for super
+        line = line.replace(
+            "super() argument 1 must be a type, not NoneType",
+            "super() argument 1 must be type, not None",
+        )
+        line = line.replace(
+            "super() argument 1 must be a type", "super() argument 1 must be type"
+        )
+
         result.append(line)
 
     return result
@@ -233,16 +229,16 @@ exceeded while calling a Python object' in \
 def compareOutput(
     kind, out_cpython, out_nuitka, ignore_warnings, syntax_errors, trace_result=True
 ):
-    fromdate = ""
-    todate = ""
+    from_date = ""
+    to_date = ""
 
     diff = difflib.unified_diff(
         makeDiffable(out_cpython, ignore_warnings, syntax_errors),
         makeDiffable(out_nuitka, ignore_warnings, syntax_errors),
         "{program} ({detail})".format(program=os.environ["PYTHON"], detail=kind),
         "{program} ({detail})".format(program="nuitka", detail=kind),
-        fromdate,
-        todate,
+        from_date,
+        to_date,
         n=3,
     )
 
